==> retro_data_structures-0.28.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/SurfaceOrientation.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection

if typing.TYPE_CHECKING:
    class SurfaceOrientationJson(typing_extensions.TypedDict):
        flags_surface_orientation: int
    

_FAST_FORMAT = None
_FAST_IDS = (0x10e6f366)


@dataclasses.dataclass()
class SurfaceOrientation(BaseProperty):
    flags_surface_orientation: int = dataclasses.field(default=256, metadata={
        'reflection': FieldReflection[int](
            int, id=0x10e6f366, original_name='FlagsSurfaceOrientation'
        ),
    })  # Flagset

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 1:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHL')
    
        dec = _FAST_FORMAT.unpack(data.read(10))
        assert (dec[0]) == _FAST_IDS
        return cls(
            dec[2],
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x01')  # 1 properties

        data.write(b'\x10\xe6\xf3f')  # 0x10e6f366
        data.write(b'\x00\x04')  # size
        data.write(struct.pack(">L", self.flags_surface_orientation))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("SurfaceOrientationJson", data)
        return cls(
            flags_surface_orientation=json_data['flags_surface_orientation'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'flags_surface_orientation': self.flags_surface_orientation,
        }


def _decode_flags_surface_orientation(data: typing.BinaryIO, property_size: int):
    return struct.unpack(">L", data.read(4))[0]


_property_decoder: typing.Dict[int, typing.Tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0x10e6f366: ('flags_surface_orientation', _decode_flags_surface_orientation),
}

==> retro_data_structures-0.29.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/SurfaceOrientation.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection

if typing.TYPE_CHECKING:
    class SurfaceOrientationJson(typing_extensions.TypedDict):
        flags_surface_orientation: int
    

_FAST_FORMAT: struct.Struct | None = None
_FAST_IDS = (0x10e6f366)


@dataclasses.dataclass()
class SurfaceOrientation(BaseProperty):
    flags_surface_orientation: int = dataclasses.field(default=256, metadata={
        'reflection': FieldReflection[int](
            int, id=0x10e6f366, original_name='FlagsSurfaceOrientation'
        ),
    })  # Flagset

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 1:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHL')
    
        dec = _FAST_FORMAT.unpack(data.read(10))
        assert (dec[0]) == _FAST_IDS
        return cls(
            dec[2],
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x01')  # 1 properties

        data.write(b'\x10\xe6\xf3f')  # 0x10e6f366
        data.write(b'\x00\x04')  # size
        data.write(struct.pack(">L", self.flags_surface_orientation))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("SurfaceOrientationJson", data)
        return cls(
            flags_surface_orientation=json_data['flags_surface_orientation'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'flags_surface_orientation': self.flags_surface_orientation,
        }


def _decode_flags_surface_orientation(data: typing.BinaryIO, property_size: int) -> int:
    return struct.unpack(">L", data.read(4))[0]


_property_decoder: dict[int, tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0x10e6f366: ('flags_surface_orientation', _decode_flags_surface_orientation),
}
